class CDescriptorTables
{
	public:
		struct gdt_entry_struct
		{
			unsigned short limit_low;           // The lower 16 bits of the limit.
			unsigned short base_low;            // The lower 16 bits of the base.
			unsigned char  base_middle;         // The next 8 bits of the base.
			unsigned char  access;              // Access flags, determine what ring this segment can be used in.
			unsigned char  granularity;
			unsigned char  base_high;           // The last 8 bits of the base.
		} __attribute__((packed));
		typedef struct gdt_entry_struct gdt_entry_t;

		struct gdt_ptr_struct
		{
			unsigned short limit;               // The upper 16 bits of all selector limits.
			unsigned int base;                // The address of the first gdt_entry_t struct.
		} __attribute__((packed));
		typedef struct gdt_ptr_struct gdt_ptr_t;

		struct idt_entry_struct
		{
			unsigned short base_lo;             // The lower 16 bits of the address to jump to when this interrupt fires.
			unsigned short sel;                 // Kernel segment selector.
			unsigned char always0;             // This must always be zero.
			unsigned char flags;               // More flags. See documentation.
			unsigned short base_hi;             // The upper 16 bits of the address to jump to.
		} __attribute__((packed));
		typedef struct idt_entry_struct idt_entry_t;

		struct idt_ptr_struct
		{
			unsigned short limit;
			unsigned int base;                // The address of the first element in our idt_entry_t array.
		} __attribute__((packed));
		typedef struct idt_ptr_struct idt_ptr_t;

		static void init_descriptor_tables();
	private:
		static void init_gdt();
		static void gdt_set_gate(int num,unsigned int base,unsigned int limit,unsigned char access,unsigned char gran);
		
		static void init_idt();
		static void idt_set_gate(unsigned short num, unsigned int base, unsigned short sel, unsigned char flags);

		static gdt_entry_t gdt_entries[5];
		static gdt_ptr_t gdt_ptr;

		static idt_entry_t idt_entries[256];
		static idt_ptr_t idt_ptr;
};