#include <PIT.h>
#include <Common.h>
#include <TextMode_X86.h>

unsigned int CPIT::tick = 0;

void CPIT::timer_callback(registers_t regs)
{
   tick++;
   CTextMode_X86::printf("Tick: %d\n", tick);
}

void CPIT::init_timer(unsigned int frequency)
{
   // Firstly, register our timer callback.
   register_interrupt_handler(IRQ0, &timer_callback);

   // The value we send to the PIT is the value to divide it's input clock
   // (1193180 Hz) by, to get our required frequency. Important to note is
   // that the divisor must be small enough to fit into 16-bits.
   unsigned int divisor = 1193180 / frequency;

   // Send the command byte.
   CCommon::outb(0x43, 0x36);

   // Divisor has to be sent byte-wise, so split here into upper/lower bytes.
   unsigned short l = (unsigned char)(divisor & 0xFF);
   unsigned short h = (unsigned char)( (divisor>>8) & 0xFF );

   // Send the frequency divisor.
   CCommon::outb(0x40, l);
   CCommon::outb(0x40, h);
}